@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/python3
 
 from installer import InstallerEngine, Setup, NON_LATIN_KB_LAYOUTS
 from dialogs import MessageDialog, QuestionDialog, ErrorDialog, WarningDialog
@@ -7,12 +7,13 @@ import partitioning
 import gettext
 import os
 import re
-import commands
+import subprocess
 import sys
 import threading
 import time
 import parted
 import cairo
+import requests
 
 import gi
 gi.require_version('Gtk', '3.0')
@@ -21,7 +22,20 @@ from gi.repository import Gtk, Gdk, GdkPixbuf, GObject, WebKit2, Pango, GLib
 
 gettext.install("live-installer", "/usr/share/locale")
 
-LOADING_ANIMATION = '/usr/share/live-installer/loading.gif'
+USERNAME_REGEX = r"[a-z_]{1}[a-z0-9_-]*[$]?"
+USERNAME_LENGTH = (1, 32)
+
+# "[[:ascii:]]+" gives nested set warning. https://www.regular-expressions.info/posixbrackets.html
+PASSWORD_REGEX = r"[\x00-\x7F]+"
+PASSWORD_LENGTHS = (1, 2147483647)
+
+HOSTNAME_REGEX = r"^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9])$"
+HOSTNAME_MIN_LENGTH = 1
+HOSTNAME_SEG_LENGTH = 63
+HOSTNAME_TOTAL_LENGTH = 253
+
+WHITESPACE_REGEX = r"[\s]+"
+HAS_LOWER_REGEX = r"[A-Z]+"
 
 # Used as a decorator to run things in the background
 def asynchronous(func):
@@ -55,9 +69,7 @@ class InstallerWindow:
 
         self.expert_mode = expert_mode
 
-        # disable the screensaver
-        if not __debug__:
-            os.system("killall cinnamon-screen")
+        Gtk.Settings.get_default().set_property("gtk-application-prefer-dark-theme", True)
 
         # build the setup object (where we put all our choices) and the installer
         self.setup = Setup()
@@ -79,8 +91,7 @@ class InstallerWindow:
         self.window.connect("delete-event", self.quit_cb)
 
         # wizard pages
-        (self.PAGE_WELCOME,
-         self.PAGE_LANGUAGE,
+        (self.PAGE_LANGUAGE,
          self.PAGE_TIMEZONE,
          self.PAGE_KEYBOARD,
          self.PAGE_USER,
@@ -90,7 +101,7 @@ class InstallerWindow:
          self.PAGE_OVERVIEW,
          self.PAGE_CUSTOMWARNING,
          self.PAGE_CUSTOMPAUSED,
-         self.PAGE_INSTALL) = range(12)
+         self.PAGE_INSTALL) = range(11)
 
         # set the button events (wizard_cb)
         self.builder.get_object("button_next").connect("clicked", self.wizard_cb, False)
@@ -108,7 +119,7 @@ class InstallerWindow:
         self.builder.get_object("treeview_language_list").append_column(self.language_column)
 
         ren = Gtk.CellRendererText()
-        self.country_column = Gtk.TreeViewColumn(_("Country"), ren, text=1)
+        self.country_column = Gtk.TreeViewColumn(_("Location"), ren, text=1)
         self.country_column.set_sort_column_id(1)
         self.country_column.set_expand(True)
         self.country_column.set_resizable(True)
@@ -162,7 +173,7 @@ class InstallerWindow:
 
         self.builder.get_object("entry_name").connect("notify::text", self.assign_realname)
         self.builder.get_object("entry_username").connect("notify::text", self.assign_username)
-        self.builder.get_object("entry_hostname").connect("notify::text", self.assign_hostname)
+        self.hostname_handler_id = self.builder.get_object("entry_hostname").connect("notify::text", self.assign_hostname)
 
         # events for detecting password mismatch..
         self.builder.get_object("entry_password").connect("changed", self.assign_password)
@@ -226,9 +237,6 @@ class InstallerWindow:
         # build partition list
         self.should_pulse = False
 
-        # make sure we're on the right page (no pun.)
-        self.activate_page(0)
-
         # initiate the slide show
         # We have no significant browsing interface, so there isn't much point
         # in WebKit creating a memory-hungry cache.
@@ -238,37 +246,48 @@ class InstallerWindow:
         self.partitions_browser = WebKit2.WebView()
         s = self.partitions_browser.get_settings()
         s.set_allow_file_access_from_file_urls(True)
+        self.partitions_browser.connect('context-menu', self.on_context_menu)
         self.partitions_browser.show()
         self.partitions_browser.set_size_request(-1, 80)
         self.builder.get_object("scrolled_partitions").add(self.partitions_browser)
 
+        self.builder.get_object("button_lets_go").connect("clicked", self.on_lets_go_clicked)
+
         self.window.show_all()
 
+    def on_lets_go_clicked(self, button):
+        self.activate_page(self.PAGE_LANGUAGE)
+        self.builder.get_object("stack").set_visible_child_name("page_notebook")
+        GObject.timeout_add(400, self.remove_dark_mode)
+
+    def remove_dark_mode(self):
+        Gtk.Settings.get_default().set_property("gtk-application-prefer-dark-theme", False)
+
     def on_context_menu(self, unused_web_view, unused_context_menu,
                     unused_event, unused_hit_test_result):
         # True will not show the menu
         return True
 
     def i18n(self):
-
-        window_title = _("Installer")
+        subtitle = _("Installer")
         with open("/etc/lsb-release") as f:
             config = dict([line.strip().split("=") for line in f])
-            window_title = "%s - %s" % (config['DISTRIB_DESCRIPTION'].replace('"', ''), _("Installer"))
+            title = config['DISTRIB_DESCRIPTION'].replace('"', '')
 
         if __debug__:
-            window_title += ' (debug)'
+            subtitle += ' - debug mode'
         self.builder.get_object("button_expert").set_no_show_all(True)
         if self.expert_mode:
-            window_title += ' (expert mode)'
+            subtitle += ' - expert mode'
             self.builder.get_object("button_expert").show()
         else:
             self.builder.get_object("button_expert").hide()
-        self.window.set_title(window_title)
+
+        self.builder.get_object("headerbar").set_title("LMDE 5 'Elsie'")
+        self.builder.get_object("headerbar").set_subtitle(subtitle)
 
         # Header
-        self.wizard_pages = range(12)
-        self.wizard_pages[self.PAGE_WELCOME] = WizardPage(_("Welcome"), "mark-location-symbolic", "")
+        self.wizard_pages = list(range(11))
         self.wizard_pages[self.PAGE_LANGUAGE] = WizardPage(_("Language"), "preferences-desktop-locale-symbolic", _("What language would you like to use?"))
         self.wizard_pages[self.PAGE_TIMEZONE] = WizardPage(_("Timezone"), "mark-location-symbolic", _("Where are you?"))
         self.wizard_pages[self.PAGE_KEYBOARD] = WizardPage(_("Keyboard layout"), "preferences-desktop-keyboard-symbolic", _("What is your keyboard layout?"))
@@ -287,19 +306,20 @@ class InstallerWindow:
         self.builder.get_object("button_next").set_label(_("Next"))
 
         # Welcome page
-        self.builder.get_object("label_welcome1").set_text(_("Welcome to the LMDE Installer."))
+        self.builder.get_object("label_welcome1").set_text(_("Welcome to %s!") % "LMDE 5")
         self.builder.get_object("label_welcome2").set_text(_("This program will ask you some questions and set up LMDE on your computer."))
+        self.builder.get_object("button_lets_go").set_label(_("Let's go!"))
 
         # Language page
         self.language_column.set_title(_("Language"))
-        self.country_column.set_title(_("Country"))
+        self.country_column.set_title(_("Location"))
 
         # Keyboard page
         self.builder.get_object("label_kb_model").set_label(_("Keyboard Model:"))
         self.column10.set_title(_("Layout"))
         self.column11.set_title(_("Variant"))
         self.builder.get_object("entry_test_kb").set_placeholder_text(_("Type here to test your keyboard layout"))
-        self.builder.get_object("label_non_latin").set_text(_("* Your username, your computer's name and your password should only contain Latin characters. In addition to your selected layout, English (US) is set as the default. You can switch layouts by pressing both Ctrl keys together."))
+        self.builder.get_object("label_non_latin").set_text(_("* Your username, your computer's name and your password should only contain Latin characters. In addition to your selected layout, English (US) is set as the default. You can switch layouts by pressing Super+Space."))
 
         # User page
         self.builder.get_object("label_name").set_text(_("Your name:"))
@@ -360,6 +380,68 @@ class InstallerWindow:
         # Refresh the current title and help question in the page header
         self.activate_page(self.PAGE_LANGUAGE)
 
+    def validate_entry(self, entry, validate_regex, len_range, check_lower=False, check_match=None, also_true=True):
+        error = None
+        value = entry.props.text
+
+        while True:
+            if check_match != None and value != check_match:
+                error = _("The values do not match.")
+                break
+            if len(value) == 0:
+                error = _("This field cannot be empty.")
+                break
+            if re.search(WHITESPACE_REGEX, value):
+                error = _("This field may not contain space characters.")
+                break
+            if check_lower and re.search(HAS_LOWER_REGEX, value):
+                error = _("This field must be lower case.")
+                break
+            if len_range:
+                if len(value) < len_range[0]:
+                    error = _("The entry is too short.")
+                elif len(value) > len_range[1]:
+                    error = _("The entry is too long.")
+                    break
+
+            r = re.fullmatch(validate_regex, value)
+            if validate_regex != None and (not re.fullmatch(validate_regex, value)):
+                error = _("This field contains invalid characters.")
+                break
+            break
+
+        if error and also_true:
+            entry.set_icon_from_icon_name(Gtk.EntryIconPosition.SECONDARY, "dialog-error")
+            entry.set_icon_tooltip_text(Gtk.EntryIconPosition.SECONDARY, error)
+        else:
+            entry.set_icon_from_icon_name(Gtk.EntryIconPosition.SECONDARY, None)
+            entry.set_icon_tooltip_text(Gtk.EntryIconPosition.SECONDARY, "")
+
+        return error
+
+    def validate_user_page(self, focus_widget=False):
+        for name in ["entry_name", "entry_hostname", "entry_username", "entry_password", "entry_confirm"]:
+            entry = self.builder.get_object(name)
+            if entry.get_text() == "" or entry.get_icon_name(Gtk.EntryIconPosition.SECONDARY) == "dialog-error":
+                if focus_widget:
+                    entry.grab_focus()
+                if entry.is_visible():
+                    self.builder.get_object("button_next").set_sensitive(False)
+                return
+        self.builder.get_object("button_next").set_sensitive(True)
+# 
+    def hostname_too_long(self, hostname):
+        segments = hostname.split(".")
+
+        for segment in segments:
+            if len(segment) > HOSTNAME_SEG_LENGTH:
+                return True
+
+        if len(hostname) > HOSTNAME_TOTAL_LENGTH:
+            return True
+
+        return False
+
     def assign_realname(self, entry, prop):
         self.setup.real_name = entry.props.text
         # Try to set the username (doesn't matter if it fails)
@@ -372,63 +454,42 @@ class InstallerWindow:
             self.builder.get_object("entry_username").set_text(text)
         except:
             pass
-        if self.setup.real_name == "":
-            self.builder.get_object("check_name").hide()
-        else:
-            self.builder.get_object("check_name").show()
+
+        self.validate_entry(self.builder.get_object("entry_username"), USERNAME_REGEX, USERNAME_LENGTH, check_lower=True)
+        self.validate_user_page()
         self.setup.print_setup()
 
     def assign_username(self, entry, prop):
         self.setup.username = entry.props.text
-        errorFound = False
-        for char in self.setup.username:
-            if(char.isupper()):
-                errorFound = True
-                break
-            elif(char.isspace()):
-                errorFound = True
-                break
-        if errorFound or self.setup.username == "":
-            self.builder.get_object("check_username").hide()
-        else:
-            self.builder.get_object("check_username").show()
+
+        self.validate_entry(entry, USERNAME_REGEX, USERNAME_LENGTH, check_lower=True)
+        self.validate_user_page()
         self.setup.print_setup()
 
     def assign_hostname(self, entry, prop):
         self.setup.hostname = entry.props.text
-        errorFound = False
-        for char in self.setup.hostname:
-            if(char.isupper()):
-                errorFound = True
-                break
-            elif(char.isspace()):
-                errorFound = True
-                break
-        if errorFound or self.setup.hostname == "":
-            self.builder.get_object("check_hostname").hide()
+
+        # Hostname is too complicated for validate_entry, as it can have multiple domains separated by a '.',
+        # and there are length rules for each segment and the total length. Do the check here, then pass along
+        # such length requirements as to trigger too short/long warnings.
+        if self.hostname_too_long(self.setup.hostname):
+            self.validate_entry(entry, HOSTNAME_REGEX, (0, 0), check_lower=True)
         else:
-            self.builder.get_object("check_hostname").show()
+            self.validate_entry(entry, HOSTNAME_REGEX, (HOSTNAME_MIN_LENGTH, 9999), check_lower=True)
+
+        self.validate_user_page()
         self.setup.print_setup()
 
     def assign_password(self, widget):
         self.setup.password1 = self.builder.get_object("entry_password").get_text()
         self.setup.password2 = self.builder.get_object("entry_confirm").get_text()
 
-        if self.setup.password1 == "":
-            self.builder.get_object("check_password").hide()
-        else:
-            self.builder.get_object("check_password").show()
-
-        # Check the password confirmation
-        if(self.setup.password1 == "" or self.setup.password2 == "" or self.setup.password1 != self.setup.password2):
-            self.builder.get_object("check_confirm").hide()
-        else:
-            self.builder.get_object("check_confirm").show()
-
+        self.validate_entry(self.builder.get_object("entry_password"), PASSWORD_REGEX, PASSWORD_LENGTHS)
+        self.validate_entry(self.builder.get_object("entry_confirm"), PASSWORD_REGEX, PASSWORD_LENGTHS, check_match=self.setup.password1)
+        self.validate_user_page()
         self.setup.print_setup()
 
-
-    def assign_type_options(self, widget, data=None):
+    def assign_type_options(self, widget=None, data=None):
         self.setup.automated = self.builder.get_object("radio_automated").get_active()
         self.builder.get_object("check_badblocks").set_sensitive(self.setup.automated)
         self.builder.get_object("check_encrypt").set_sensitive(self.setup.automated)
@@ -477,13 +538,32 @@ class InstallerWindow:
 
         self.setup.badblocks = self.builder.get_object("check_badblocks").get_active()
 
+        block_next = False
+
+        if self.setup.automated:
+            block_next = block_next or self.setup.disk == None
+            if self.builder.get_object("check_encrypt").get_active():
+                block_next = block_next or self.setup.passphrase1 in (None, "") or self.setup.passphrase1 != self.setup.passphrase2
+
+        self.builder.get_object("button_next").set_sensitive(not block_next)
         self.setup.print_setup()
 
     def assign_passphrase(self, widget):
         self.setup.passphrase1 = self.builder.get_object("entry_passphrase").get_text()
         self.setup.passphrase2 = self.builder.get_object("entry_passphrase2").get_text()
+
+        self.validate_entry(self.builder.get_object("entry_passphrase"), PASSWORD_REGEX, PASSWORD_LENGTHS, also_true=self.setup.luks)
+        self.validate_entry(self.builder.get_object("entry_passphrase2"), PASSWORD_REGEX, PASSWORD_LENGTHS, check_match=self.setup.passphrase1, also_true=self.setup.luks)
+
+        self.update_disk_selection_next()
         self.setup.print_setup()
 
+    def update_disk_selection_next(self):
+        error = self.setup.disk == None
+        error = error or self.setup.luks and (self.setup.passphrase1 in (None, "") or self.setup.passphrase1 != self.setup.passphrase2)
+
+        self.builder.get_object("button_next").set_sensitive(not error)
+
     def quit_cb(self, widget, data=None):
         if QuestionDialog(_("Quit?"), _("Are you sure you want to quit the installer?")):
             Gtk.main_quit()
@@ -494,15 +574,20 @@ class InstallerWindow:
     def show_customwarning(self, widget):
         self.activate_page(self.PAGE_CUSTOMWARNING)
 
+    def flag(self, lang, language, ccode):
+        print (language.lower())
+        for filename in [language.lower(), ccode.lower(), "united_nations"]:
+            path = f"/usr/share/circle-flags-svg/{filename}.svg"
+            if os.path.exists(path):
+                break
+        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(path, 18, 18)
+        return pixbuf
+
     def build_lang_list(self):
 
         # Try to find out where we're located...
         try:
-            from urllib import urlopen
-        except ImportError:  # py3
-            from urllib.request import urlopen
-        try:
-            lookup = str(urlopen('http://geoip.ubuntu.com/lookup').read())
+            lookup = requests.get('http://geoip.ubuntu.com/lookup', timeout=5).content.decode("UTF-8")
             self.cur_country_code = re.search('<CountryCode>(.*)</CountryCode>', lookup).group(1)
             self.cur_timezone = re.search('<TimeZone>(.*)</TimeZone>', lookup).group(1)
             if self.cur_country_code == 'None': self.cur_country_code = "US"
@@ -515,7 +600,7 @@ class InstallerWindow:
         iso_standard = "3166"
         if os.path.exists("/usr/share/xml/iso-codes/iso_3166-1.xml"):
             iso_standard = "3166-1"
-        for line in commands.getoutput("isoquery --iso %s | cut -f1,4-" % iso_standard).split('\n'):
+        for line in subprocess.getoutput("isoquery --iso %s | cut -f1,4-" % iso_standard).split('\n'):
             ccode, cname = line.split(None, 1)
             countries[ccode] = cname
 
@@ -524,25 +609,22 @@ class InstallerWindow:
         iso_standard = "639"
         if os.path.exists("/usr/share/xml/iso-codes/iso_639-2.xml"):
             iso_standard = "639-2"
-        for line in commands.getoutput("isoquery --iso %s | cut -f3,4-" % iso_standard).split('\n'):
+        for line in subprocess.getoutput("isoquery --iso %s | cut -f3,4-" % iso_standard).split('\n'):
             cols = line.split(None, 1)
             if len(cols) > 1:
                 name = cols[1].replace(";", ",")
                 languages[cols[0]] = name
-        for line in commands.getoutput("isoquery --iso %s | cut -f1,4-" % iso_standard).split('\n'):
+        for line in subprocess.getoutput("isoquery --iso %s | cut -f1,4-" % iso_standard).split('\n'):
             cols = line.split(None, 1)
             if len(cols) > 1:
-                if cols[0] not in languages.keys():
+                if cols[0] not in languages:
                     name = cols[1].replace(";", ",")
                     languages[cols[0]] = name
 
         # Construct language selection model
         model = Gtk.ListStore(str, str, GdkPixbuf.Pixbuf, str)
         set_iter = None
-        flag_path = lambda ccode: self.resource_dir + '/flags/16/' + ccode.lower() + '.png'
-        from utils import memoize
-        flag = memoize(lambda ccode: GdkPixbuf.Pixbuf.new_from_file(flag_path(ccode)))
-        for locale in commands.getoutput("awk -F'[@ .]' '/UTF-8/{ print $1 }' /usr/share/i18n/SUPPORTED | uniq").split('\n'):
+        for locale in subprocess.getoutput("awk -F'[@ .]' '/UTF-8/{ print $1 }' /usr/share/i18n/SUPPORTED | uniq").split('\n'):
             if '_' in locale:
                 lang, ccode = locale.split('_')
                 language = lang
@@ -550,7 +632,8 @@ class InstallerWindow:
                 try:
                     language = languages[lang]
                 except:
-                    pass
+                    print(f"Skipping unknown language code '{lang}'")
+                    continue
                 try:
                     country = countries[ccode]
                 except:
@@ -560,9 +643,14 @@ class InstallerWindow:
                 try:
                     language = languages[lang]
                 except:
-                    pass
+                    print(f"Skipping unknown language code '{lang}'")
+                    continue
                 country = ''
-            pixbuf = flag(ccode) if not lang in 'eo ia' else flag('_' + lang)
+            language = language.split(",")[0].split("(")[0]
+            country = country.split(",")[0].split("(")[0]
+            if country == '':
+                continue
+            pixbuf = self.flag(lang, language, ccode)
             iter = model.append((language, country, pixbuf, locale))
             if (ccode == self.cur_country_code and
                 (not set_iter or
@@ -585,7 +673,7 @@ class InstallerWindow:
         ''' Do some xml kung-fu and load the keyboard stuffs '''
         # Determine the layouts in use
         (keyboard_geom,
-         self.setup.keyboard_layout) = commands.getoutput("setxkbmap -query | awk '/^(model|layout)/{print $2}'").split()
+         self.setup.keyboard_layout) = subprocess.getoutput("setxkbmap -query | awk '/^(model|layout)/{print $2}'").split()
         # Build the models
         from collections import defaultdict
         def _ListStore_factory():
@@ -641,7 +729,7 @@ class InstallerWindow:
         ''' Called whenever someone updates the language '''
         model = treeview.get_model()
         selection = treeview.get_selection()
-        if selection.count_selected_rows > 0:
+        if selection.count_selected_rows() > 0:
             (model, iter) = selection.get_selected()
             if iter is not None:
                 self.setup.language = model.get_value(iter, 3)
@@ -731,7 +819,7 @@ class InstallerWindow:
         else:
             self.builder.get_object("label_non_latin").hide()
 
-        command = "setxkbmap -layout '%s' -variant '%s' -option grp:ctrls_toggle" % (self.setup.keyboard_layout, self.setup.keyboard_variant)
+        command = "setxkbmap -layout '%s' -variant '%s' -option grp:win_space_toggle" % (self.setup.keyboard_layout, self.setup.keyboard_variant)
         if not __debug__:
             os.system(command)
             self.setup.print_setup()
@@ -756,7 +844,7 @@ class InstallerWindow:
         else:
             hidpi = "normal"
 
-        os.system("python /usr/lib/live-installer/frontend/generate_keyboard_layout.py %s %s %s %s" % (layout, variant, filename, hidpi))
+        os.system("/usr/lib/live-installer/frontend/generate_keyboard_layout.py %s %s %s %s" % (layout, variant, filename, hidpi))
         self._on_layout_generated()
 
     @idle
@@ -778,8 +866,9 @@ class InstallerWindow:
         return False
 
     def activate_page(self, index):
+
         # progress images
-        for i in range(9):
+        for i in range(8):
             img = self.builder.get_object("progress_%d" % i)
             if i <= index:
                 img.set_from_file("/usr/share/icons/live-installer-progress-dot-on.png")
@@ -791,6 +880,10 @@ class InstallerWindow:
         self.builder.get_object("help_question").set_text(self.wizard_pages[index].question)
         self.builder.get_object("notebook1").set_current_page(index)
         # TODO: move other page-depended actions from the wizard_cb into here below
+        if index == self.PAGE_LANGUAGE:
+            self.builder.get_object("button_back").hide()
+        else:
+            self.builder.get_object("button_back").show()
         if index == self.PAGE_PARTITIONS:
             self.setup.skip_mount = False
         if index == self.PAGE_CUSTOMWARNING:
@@ -803,9 +896,7 @@ class InstallerWindow:
 
         # check each page for errors
         if(not goback):
-            if (sel == self.PAGE_WELCOME):
-                self.activate_page(self.PAGE_LANGUAGE)
-            elif(sel == self.PAGE_LANGUAGE):
+            if(sel == self.PAGE_LANGUAGE):
                 if self.setup.language is None:
                     WarningDialog(_("Installer"), _("Please choose a language"))
                 else:
@@ -841,83 +932,16 @@ class InstallerWindow:
                 self.activate_page(self.PAGE_KEYBOARD)
             elif(sel == self.PAGE_KEYBOARD):
                 self.activate_page(self.PAGE_USER)
-                self.builder.get_object("entry_name").grab_focus()
+                self.validate_user_page(focus_widget=True)
             elif(sel == self.PAGE_USER):
-                errorFound = False
-                errorMessage = ""
-                focus_widget = None
-
-                if(self.setup.real_name is None or self.setup.real_name == ""):
-                    errorFound = True
-                    errorMessage = _("Please provide your full name.")
-                    focus_widget = self.builder.get_object("entry_name")
-                elif(self.setup.hostname is None or self.setup.hostname == ""):
-                    errorFound = True
-                    errorMessage = _("Please provide a name for your computer.")
-                    focus_widget = self.builder.get_object("entry_hostname")
-                elif(self.setup.username is None or self.setup.username == ""):
-                    errorFound = True
-                    errorMessage = _("Please provide a username.")
-                    focus_widget = self.builder.get_object("entry_username")
-                elif(self.setup.password1 is None or self.setup.password1 == ""):
-                    errorFound = True
-                    errorMessage = _("Please provide a password for your user account.")
-                    focus_widget = self.builder.get_object("entry_password")
-                elif(self.setup.password1 != self.setup.password2):
-                    errorFound = True
-                    errorMessage = _("Your passwords do not match.")
-                    focus_widget = self.builder.get_object("entry_confirm")
-                else:
-                    for char in self.setup.username:
-                        if(char.isupper()):
-                            errorFound = True
-                            errorMessage = _("Your username must be lower case.")
-                            focus_widget = self.builder.get_object("entry_username")
-                            break
-                        elif(char.isspace()):
-                            errorFound = True
-                            errorMessage = _("Your username may not contain whitespace characters.")
-                            focus_widget = self.builder.get_object("entry_username")
-                            break
-                    for char in self.setup.hostname:
-                        if(char.isupper()):
-                            errorFound = True
-                            errorMessage = _("The computer's name must be lower case.")
-                            focus_widget = self.builder.get_object("entry_hostname")
-                            break
-                        elif(char.isspace()):
-                            errorFound = True
-                            errorMessage = _("The computer's name may not contain whitespace characters.")
-                            focus_widget = self.builder.get_object("entry_hostname")
-                            break
-
-                if (errorFound):
-                    WarningDialog(_("Installer"), errorMessage)
-                    if focus_widget is not None:
-                        focus_widget.grab_focus()
-                else:
-                    self.activate_page(self.PAGE_TYPE)
+                self.activate_page(self.PAGE_TYPE)
+                self.assign_type_options()
             elif(sel == self.PAGE_TYPE):
                 if self.setup.automated:
-                    errorFound = False
-                    errorMessage = ""
-                    if self.setup.disk is None:
-                        errorFound = True
-                        errorMessage = _("Please select a disk.")
-                    if self.setup.luks:
-                        if (self.setup.passphrase1 is None or self.setup.passphrase1 == ""):
-                            errorFound = True
-                            errorMessage = _("Please provide a passphrase for the encryption.")
-                        elif (self.setup.passphrase1 != self.setup.passphrase1):
-                            errorFound = True
-                            errorMessage = _("Your passphrases do not match.")
-                    if (errorFound):
-                        WarningDialog(_("Installer"), errorMessage)
-                    else:
-                        if QuestionDialog(_("Warning"), _("This will delete all the data on %s. Are you sure?") % self.setup.diskname):
-                            partitioning.build_partitions(self)
-                            partitioning.build_grub_partitions()
-                            self.activate_page(self.PAGE_ADVANCED)
+                    if QuestionDialog(_("Warning"), _("This will delete all the data on %s. Are you sure?") % self.setup.diskname):
+                        partitioning.build_partitions(self)
+                        partitioning.build_grub_partitions()
+                        self.activate_page(self.PAGE_ADVANCED)
                 else:
                     self.activate_page(self.PAGE_PARTITIONS)
                     partitioning.build_partitions(self)
@@ -932,27 +956,12 @@ class InstallerWindow:
                         if partition.format_as is None or partition.format_as == "":
                             ErrorDialog(_("Installer"), _("Please indicate a filesystem to format the root (/) partition with before proceeding."))
                             return
-                    if partition.mount_as == "/@":
-                        if partition.format_as != "btrfs":
-                            ErrorDialog(_("Installer"), _("A root subvolume (/@) requires to format the partition with btrfs."))
-                            return
-                        found_root_partition = True
-                    if partition.mount_as == "/@home":
-                        if partition.format_as == "btrfs":
-                            continue;
-                        if partition.type == "btrfs" and (partition.format_as == None or partition.format_as == ""):
-                            continue;
-                        ErrorDialog(_("Installer"), _("A home subvolume (/@home) requires the use of a btrfs formatted partition."))
-                        return
 
                 if not found_root_partition:
                     ErrorDialog(_("Installer"), "<b>%s</b>" % _("Please select a root (/) partition."), _(
                         "A root partition is needed to install Linux Mint on.\n\n"
                         " - Mount point: /\n - Recommended size: 30GB\n"
-                        " - Recommended filesystem format: ext4\n\n"
-                        "Note: The timeshift btrfs snapshots feature requires the use of:\n"
-                        " - subvolume Mount-point /@\n"
-                        " - btrfs as filesystem format\n"))
+                        " - Recommended filesystem format: ext4\n"))
                     return
 
                 if self.setup.gptonefi:
@@ -1020,6 +1029,7 @@ class InstallerWindow:
                 self.paused = False
         else:
             self.builder.get_object("button_back").set_sensitive(True)
+            self.builder.get_object("button_next").set_sensitive(True)
             if(sel == self.PAGE_OVERVIEW):
                 self.activate_page(self.PAGE_ADVANCED)
             elif(sel == self.PAGE_ADVANCED):
@@ -1041,8 +1051,6 @@ class InstallerWindow:
                 self.activate_page(self.PAGE_TIMEZONE)
             elif(sel == self.PAGE_TIMEZONE):
                 self.activate_page(self.PAGE_LANGUAGE)
-            elif(sel == self.PAGE_LANGUAGE):
-                self.activate_page(self.PAGE_WELCOME)
 
     def show_overview(self):
         bold = lambda str: '<b>' + str + '</b>'
@@ -1105,11 +1113,17 @@ class InstallerWindow:
 
     @asynchronous
     def do_install(self):
-        print " ## INSTALLATION "
-        ''' Actually perform the installation .. '''
+        print(" ## INSTALLATION ")
+        print(' Actually perform the installation .. ')
 
         if __debug__:
-            print " ## DEBUG MODE - INSTALLATION PROCESS NOT LAUNCHED"
+            print(" ## DEBUG MODE - INSTALLATION PROCESS NOT LAUNCHED")
+            for partition in self.setup.partitions:
+                if(partition.format_as is not None and partition.format_as != ""):
+                    partition.type = partition.format_as
+            self.installer.write_fstab("dummy_fstab")
+            self.installer.write_mtab("dummy_fstab", "dummy_mtab")
+            self.installer.write_crypttab("dummy_crypttab")
             time.sleep(200)
             Gtk.main_quit()
             sys.exit(0)
@@ -1125,8 +1139,8 @@ class InstallerWindow:
 
         try:
             self.installer.start_installation()
-        except Exception, detail1:
-            print detail1
+        except Exception as detail1:
+            print(detail1)
             do_try_finish_install = False
             self.show_error_dialog(_("Installation error"), str(detail1))
 
@@ -1143,8 +1157,8 @@ class InstallerWindow:
 
             try:
                 self.installer.finish_installation()
-            except Exception, detail1:
-                print detail1
+            except Exception as detail1:
+                print(detail1)
                 self.show_error_dialog(_("Installation error"), str(detail1))
 
             # show a message dialog thingum
@@ -1160,7 +1174,7 @@ class InstallerWindow:
             while(self.showing_last_dialog):
                 time.sleep(0.1)
 
-            print " ## INSTALLATION COMPLETE "
+            print(" ## INSTALLATION COMPLETE ")
 
         Gtk.main_quit()
         sys.exit(0)
